# -*- coding: utf-8 -*
import sys

import glob
import json
import os
import os.path
import pandas as pd
import requests
import shutil
import zipfile

from flyai.utils import read_data
from flyai.utils.qiniu_helper import Qiniu
from sklearn.model_selection import train_test_split


def dataset_split(from_dir,
                  columns=None,
                  helpers=None,
                  to_dir='output/',
                  test_size=0.2,
                  validation_size=0.2,
                  shuffle=True,
                  seed=None):
    # from_dir = os.path.join(sys.path[0], from_dir)
    csvs = glob.glob(from_dir + '*.csv')
    print(csvs)
    isHeader = True
    for csv in csvs:
        df = read_data.read(csv)
        df.to_csv(from_dir + "flyai_all.csv", mode='a', index=False, header=isHeader)
        isHeader = False

    df = read_data.read(from_dir + "flyai_all.csv")
    total_size = (test_size + validation_size)
    train, split = train_test_split(df, test_size=total_size, random_state=seed, shuffle=shuffle)
    validation_size = (validation_size / (test_size + validation_size))
    test, validation = train_test_split(split, test_size=validation_size, random_state=seed, shuffle=shuffle)
    to_dir = os.path.join(from_dir, to_dir)
    make_dir(to_dir)

    helper_move(helpers, from_dir, to_dir + 'train/')
    helper_move(helpers, from_dir, to_dir + 'dev/')
    helper_move(helpers, from_dir, to_dir + 'validation/')
    helper_move(helpers, from_dir, to_dir + 'test/')

    print("split train dataset")
    train.to_csv(to_dir + 'train/' + 'train.csv', index=False)

    if columns is not None:
        dataset_move(from_dir, train, columns, to_dir + 'train/')

    print("compress train to zip")
    compress_file(to_dir + 'train/', 'train', to_dir + 'train.zip')

    print("split dev dataset")
    dev = train.head(n=100)
    dev.to_csv(to_dir + 'dev/' + 'dev.csv', index=False)
    if columns is not None:
        dataset_move(from_dir, dev, columns, to_dir + 'dev/')

    print("compress dev to zip")
    compress_file(to_dir + 'dev/', 'dev', to_dir + 'dev.zip')

    print("split validation dataset")
    validation.to_csv(to_dir + 'validation/' + 'validation.csv', index=False)
    if columns is not None:
        dataset_move(from_dir, validation, columns, to_dir + 'validation/')

    print("compress validation to zip")
    compress_file(to_dir + 'validation/', 'validation', to_dir + 'validation.zip')

    print("split test dataset")
    test.to_csv(to_dir + 'test/' + 'test.csv', index=False)
    if columns is not None:
        dataset_move(from_dir, test, columns, to_dir + 'test/')

    print("compress test to zip")
    compress_file(to_dir + 'test/', 'test', to_dir + 'test.zip')

    del_file(from_dir + "flyai_all.csv")


def helper_move(helpers, from_dir, to_dir):
    if helpers is not None:
        for helper in helpers:
            copy(os.path.join(from_dir, helper), to_dir)


def dataset_move(from_dir, df, columns, to_dir=None):
    for index, row in df.iterrows():
        for column in columns:
            real_dir = to_dir + os.path.dirname(row[column])
            print(from_dir + row[column])
            print(real_dir)
            if not os.path.exists(real_dir):
                os.makedirs(real_dir)
            copy(from_dir + row[column], real_dir)


def del_file(file):
    if os.path.exists(file):  # 如果文件存在
        # 删除文件，可使用以下两种方法。
        os.remove(file)  # 则删除


def make_dir(dir):
    if not os.path.exists(dir + 'train/'):
        os.makedirs(dir + 'train/')
    if not os.path.exists(dir + 'validation/'):
        os.makedirs(dir + 'validation/')
    if not os.path.exists(dir + 'test/'):
        os.makedirs(dir + 'test/')
    if not os.path.exists(dir + 'dev/'):
        os.makedirs(dir + 'dev/')


def copy(source, target):
    shutil.copy(source, target)


def compress_file(dir, file_name, zip_name):
    if os.path.isfile(dir):
        with zipfile.ZipFile(zip_name, 'w') as z:
            z.write(dir)
    else:
        with zipfile.ZipFile(zip_name, 'w') as z:
            for root, dirs, files in os.walk(dir):
                for single_file in files:
                    if single_file != zip_name:
                        file_path = os.path.join(root, single_file)
                        real_path = file_path[file_path.find(file_name) + len(file_name):]
                        z.write(file_path, real_path)


def upload(access_key, secret_key, servers, dir, dataset_json='dataset.json'):
    f = open(os.path.join(sys.path[0], dataset_json), 'r', encoding="utf-8")
    data = f.read()
    headers = {'Content-Type': 'application/json'}
    r = requests.post(servers + "/create_dataset", headers=headers, data=data.encode('utf-8'))
    print(r.text)
    source = json.loads(r.text)
    if source['status'] == 1:
        if dir is not None:
            zips = glob.glob(dir + '*.zip')
            q = Qiniu(access_key, secret_key)
            for zip in zips:
                file_name = 'dataset/' + source['data_id'] + '/' + os.path.basename(zip)
                exist = q.exists('flyai', file_name)
                if not exist:
                    q.upload('flyai', zip, file_name)
                    print("upload file " + file_name)
                else:
                    print("file " + file_name + " exist")
